//
// Giuseppe Vacanti, August 1999
//
#include "EventAttributes.h"
#include <Tracer.h>
#include <string>
#include <map>
#include <sstream>
#include <errstr.h>

using namespace EventAttributes;


namespace {
    const string prefix = "XMMEA_";
    
    const int RGS = 0;
    const int MOS = 1;
    const int PN = 2;
    const int NOINST = 3;

    struct ExtEvAtt {
	EvAtt ea;
	std::string name;
	int instrument;
	bool info;
	ExtEvAtt(EvAtt n, std::string s, int i, bool b):
	    ea(n), name(s), instrument(i), info(b) {}
    };

    std::vector<ExtEvAtt> ExtEvAtt_init () {
	std::vector<ExtEvAtt> v;

	// RGS
	v.push_back(ExtEvAtt(EVLIST_LOCAL, "EVLIST_LOCAL", RGS, true));
	v.push_back(ExtEvAtt(ON_NODE0, "ON_NODE0", RGS, true));
	v.push_back(ExtEvAtt(ON_NODE1, "ON_NODE1", RGS, true));
	v.push_back(ExtEvAtt(ONBOARD_SER, "ONBOARD_SER", RGS, true));
	v.push_back(ExtEvAtt(FLICKERING, "FLICKERING", RGS, true));
	v.push_back(ExtEvAtt(ON_NODE_INTERFACE, "ON_NODE_INTERFACE", RGS, false));
	v.push_back(ExtEvAtt(IN_BAD_FRAME, "IN_BAD_FRAME", RGS, false));
	v.push_back(ExtEvAtt(OUTSIDE_GTI, "OUTSIDE_GTI", RGS, false));
	v.push_back(ExtEvAtt(BAD_SHAPE, "BAD_SHAPE", RGS, false));
	v.push_back(ExtEvAtt(ON_BADPIX, "ON_BADPIX", RGS, false));
	v.push_back(ExtEvAtt(NEXT_TO_CCD_BORDER, "NEXT_TO_CCD_BORDER", RGS, false));
	v.push_back(ExtEvAtt(ON_WINDOW_BORDER, "ON_WINDOW_BORDER", RGS, false));
	v.push_back(ExtEvAtt(NEXT_TO_BADPIX, "NEXT_TO_BADPIX", RGS, false));
	v.push_back(ExtEvAtt(BELOW_ACCEPTANCE, "BELOW_ACCEPTANCE", RGS, false));
	v.push_back(ExtEvAtt(ON_BADROW, "ON_BADROW", RGS, false));

	// MOS
	v.push_back(ExtEvAtt(EVLIST_LOCAL, "EVLIST_LOCAL", MOS, true));
	v.push_back(ExtEvAtt(DIAGONAL, "DIAGONAL", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_CCD_BORDER, "CLOSE_TO_CCD_BORDER", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_CCD_WINDOW, "CLOSE_TO_CCD_WINDOW", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_NODE_BOUNDARY, "CLOSE_TO_NODE_BOUNDARY", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_ONBOARD_BADPIX, "CLOSE_TO_ONBOARD_BADPIX", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_BRIGHTPIX, "CLOSE_TO_BRIGHTPIX", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_DEADPIX, "CLOSE_TO_DEADPIX", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_BADCOL, "CLOSE_TO_BADCOL", MOS, true));
	v.push_back(ExtEvAtt(CLOSE_TO_BADROW, "CLOSE_TO_BADROW", MOS, true));
	v.push_back(ExtEvAtt(IN_SPOILED_FRAME, "IN_SPOILED_FRAME", MOS, true));
	v.push_back(ExtEvAtt(UNKNOWN_RAWX_PATTERN, "UNKNOWN_RAWX_PATTERN", MOS, true));
	v.push_back(ExtEvAtt(ON_BADOFFSET, "ON_BADOFFSET", MOS, false));
	v.push_back(ExtEvAtt(FLICKERING, "FLICKERING", MOS, false));
	v.push_back(ExtEvAtt(OUT_OF_FOV, "OUT_OF_FOV", MOS, false));
	v.push_back(ExtEvAtt(IN_BAD_FRAME, "IN_BAD_FRAME", MOS, false));
	v.push_back(ExtEvAtt(COSMIC_RAY, "COSMIC_RAY", MOS, false));
	v.push_back(ExtEvAtt(ON_BADPIX, "ON_BADPIX", MOS, false));
	v.push_back(ExtEvAtt(REJECTED_BY_GATTI, "REJECTED_BY_GATTI", MOS, false));
	v.push_back(ExtEvAtt(OUT_OF_CCD_WINDOW, "OUT_OF_CCD_WINDOW", MOS, false));
	v.push_back(ExtEvAtt(OUTSIDE_THRESHOLDS, "OUTSIDE_THRESHOLDS", MOS, false));
	v.push_back(ExtEvAtt(BELOW_ACCEPTANCE, "BELOW_ACCEPTANCE", MOS, false));
	v.push_back(ExtEvAtt(ON_BADROW, "ON_BADROW", MOS, false));
	v.push_back(ExtEvAtt(BAD_E3E4, "BAD_E3E4", MOS, false));
	v.push_back(ExtEvAtt(UNDERSHOOT, "UNDERSHOOT", MOS, false));

	// PN
	v.push_back(ExtEvAtt(EVLIST_LOCAL, "EVLIST_LOCAL", PN, true));
	v.push_back(ExtEvAtt(INVALID_PATTERN, "INVALID_PATTERN", PN, true));
	v.push_back(ExtEvAtt(CLOSE_TO_CCD_WINDOW, "CLOSE_TO_CCD_WINDOW", PN, true));
	v.push_back(ExtEvAtt(NEXT_TO_OFFSET_COLUMN, "NEXT_TO_OFFSET_COLUMN", PN, true));
	v.push_back(ExtEvAtt(ON_OFFSET_COLUMN, "ON_OFFSET_COLUMN", PN, true));
	v.push_back(ExtEvAtt(CLOSE_TO_ONBOARD_BADPIX, "CLOSE_TO_ONBOARD_BADPIX", PN, true));
	v.push_back(ExtEvAtt(CLOSE_TO_BRIGHTPIX, "CLOSE_TO_BRIGHTPIX", PN, true));
	v.push_back(ExtEvAtt(CLOSE_TO_FLICKERINGPIX, "CLOSE_TO_FLICKERINGPIX", PN, true));
	v.push_back(ExtEvAtt(CLOSE_TO_DEADPIX, "CLOSE_TO_DEADPIX", PN, true));
	v.push_back(ExtEvAtt(CLOSE_TO_BADCOL, "CLOSE_TO_BADCOL", PN, true));
	v.push_back(ExtEvAtt(CLOSE_TO_BADROW, "CLOSE_TO_BADROW", PN, true));
	v.push_back(ExtEvAtt(IN_SPOILED_FRAME, "IN_SPOILED_FRAME", PN, true));
	v.push_back(ExtEvAtt(OUT_OF_FOV, "OUT_OF_FOV", PN, true));
	v.push_back(ExtEvAtt(FLICKERING, "FLICKERING", PN, true));
	v.push_back(ExtEvAtt(IN_BAD_FRAME, "IN_BAD_FRAME", PN, false));
	v.push_back(ExtEvAtt(COSMIC_RAY, "COSMIC_RAY", PN, false));
	v.push_back(ExtEvAtt(MIP_ASSOCIATED, "MIP_ASSOCIATED", PN, false));
	v.push_back(ExtEvAtt(ON_BADPIX, "ON_BADPIX", PN, false));
	v.push_back(ExtEvAtt(SECONDARY, "SECONDARY", PN, false));
	v.push_back(ExtEvAtt(TRAILING, "TRAILING", PN, false));
	v.push_back(ExtEvAtt(OUT_OF_CCD_WINDOW, "OUT_OF_CCD_WINDOW", PN, false));
	v.push_back(ExtEvAtt(OUTSIDE_THRESHOLDS, "OUTSIDE_THRESHOLDS", PN, false));
	v.push_back(ExtEvAtt(ON_BADROW, "ON_BADROW", PN, false));

	return v;
    }
	
    string attributeName(EvAtt n)
    {
	stringstream s;
	s << prefix << static_cast<int>(n);
	return s.str();
    }
    
    // map NAME - BIT NUMBER
    map<string, EvAtt> evatt_map() {
	map<string, EvAtt> ea;
	
	std::vector<ExtEvAtt> v = ExtEvAtt_init();
	for(std::vector<ExtEvAtt>::const_iterator K = v.begin();
	    K != v.end(); ++K){
	    const ExtEvAtt & e = *K;
	    ea[e.name] = e.ea;
	}
	return ea;
    }
    
    // map INSTRUMENT - BIT NUMBER => NAME
    string r_evatt_map(const int & i, const EvAtt n)
    {
	map<EvAtt, string> ea;

	for(int j = 0; j < NUMBER_OF; j++)
	    ea[static_cast<EvAtt>(j)] = attributeName(static_cast<EvAtt>(j));


	std::vector<ExtEvAtt> v = ExtEvAtt_init();
	for(std::vector<ExtEvAtt>::const_iterator K = v.begin();
	    K != v.end(); ++K){
	    const ExtEvAtt & e = *K;
	    if(e.instrument == i){
		ea[e.ea] = e.name;
	    }
	}

	return ea[n];
    }

    vector<string> rejectionNames()
    {
	vector<string> names;
	names.push_back(prefix+"EM");
	names.push_back(prefix+"EP");
	names.push_back(prefix+"RG");
	// SCR 198
	names.push_back(prefix+"SM");
	return names;
    }

    void propagateRejectionMasks(const Table *fromtable, Table *totable)
    {
	vector<string> names = rejectionNames();
	for(unsigned i = 0; i < names.size(); i++)
	    if(fromtable->hasAttribute(names[i]))
		*totable->addAttribute(names[i]) = *fromtable->attribute(names[i]);  
    }
  
  

};


// namespace EventAttributes implementation here


// Overloaded functions:
//  - setting an event attribute the selection expression needs !=
//  - setting the mask the selection expression needs ==
void EventAttributes::writeSelectExpression(ostream & os, EvAtt n) {
  int ea = 0;
  setEventAttribute(ea, n);
  ios::fmtflags old_options = os.flags();
  os.setf(ios::showbase);
  os.setf(ios::hex, ios::basefield);
  os << "(" + COLUMN_NAME << " & " << ea << ") != 0";
  os.flags(old_options);
}
  
void EventAttributes::writeSelectExpression(ostream & os, int mask) {
  ios::fmtflags old_options = os.flags();
  os.setf(ios::showbase);
  os.setf(ios::hex, ios::basefield);
  os << "(" + COLUMN_NAME << " & " << mask << ") == 0";
  os.flags(old_options);
}
  
void EventAttributes::writePreferenceFile(ostream & os) {
  std::vector<ExtEvAtt> v = ExtEvAtt_init ();
  os << "\"xmmselect\" {\n \"PrefSelections\" {\n";
  
  // RGS
  os << "\"RGS event attributes\" {\n";

  for(std::vector<ExtEvAtt>::const_iterator K = v.begin(); K != v.end(); ++K){
    const ExtEvAtt & e = *K;
    if(e.instrument != RGS) continue;
    os << "\"" << e.name << "\" \""; 
    writeSelectExpression(os, e.ea);
    os << "\"\n";
  }
  os << "}\n";

  // MOS
  os << "\"MOS event attributes\" {\n";

  for(std::vector<ExtEvAtt>::const_iterator K = v.begin(); K != v.end(); ++K){
    const ExtEvAtt & e = *K;
    if(e.instrument != MOS) continue;
    os << "\"" << e.name << "\" \""; 
    writeSelectExpression(os, e.ea);
    os << "\"\n";
  }
  os << "}\n";

  // PN
  os << "\"PN event attributes\" {\n";

  for(std::vector<ExtEvAtt>::const_iterator K = v.begin(); K != v.end(); ++K){
    const ExtEvAtt & e = *K;
    if(e.instrument != PN) continue;
    os << "\"" << e.name << "\" \""; 
    writeSelectExpression(os, e.ea);
    os << "\"\n";
  }
  os << "}\n";

  //
  os << "}\n}\n";
}


void EventAttributes::setEventAttribute(int & v, const EvAtt n)
{
  PROC
  v |= 1 << (int) n;
  ENDPROC
}

void EventAttributes::setEmosRejectionMask(Table *table,int mask)
{
  PROC
  string name = prefix + "EM";
  stringstream s;
  writeSelectExpression(s, mask);
  string value = s.str();
  *table->addAttribute(name, "Select good MOS events") = value;
  ENDPROC
}

void EventAttributes::setEpnRejectionMask(Table *table,int mask)
{
  PROC
  string name = prefix + "EP";
  stringstream s;
  writeSelectExpression(s, mask);
  string value = s.str();
  *table->addAttribute(name, "Select good PN events") = value;
  ENDPROC
}

void EventAttributes::setRgsRejectionMask(Table *table,int mask)
{
  PROC
  string name = prefix + "RG";
  stringstream s;
  writeSelectExpression(s, mask);
  string value = s.str();
  *table->addAttribute(name, "Select good RGS events") = value;
  ENDPROC
}

// SCR 198
void EventAttributes::setEmosSpectralMask(Table *table, int mask)
{
  PROC
  string name = prefix + "SM";
  stringstream s;
  writeSelectExpression(s, mask);
  string value = s.str();
  *table->addAttribute(name, "Select good MOS events for spectra") = value;
  ENDPROC
}


string EventAttributes::nameOfEventAttribute(const Table *table, const EvAtt n)
{
  PROC
  string s;
  if(table->parent()->hasAttribute("INSTRUME")){
    string a = table->parent()->attribute("INSTRUME")->asString();
    if(a == "RGS1" || a == "RGS2")
      s = r_evatt_map(RGS,n);
    else if(a == "EMOS1" || a == "EMOS2")
      s = r_evatt_map(MOS,n);
    else if(a == "EPN")
      s = r_evatt_map(PN,n);
    else
      s = r_evatt_map(NOINST,n);
  } else {
    s = r_evatt_map(NOINST,n);
  }
  RETURN s;
}

void EventAttributes::setEventAttribute(Table *table, const EvAtt n){
  PROC
  string name = attributeName(n);
  stringstream s;
  writeSelectExpression(s, n);
  string value = s.str();

  if(table->hasAttribute(name))
    Sas::message(Msg::Lib, Msg::Verbose, name + " exists in " + table->qualifiedName());
  else
    {
      *table->addAttribute(name, nameOfEventAttribute(table, n)) = value;
      Sas::message(Msg::Lib, Msg::Verbose, name + " (" + nameOfEventAttribute(table, n) + ") added to " + table->qualifiedName());
    }
  ENDPROC
}

void EventAttributes::copyEventAttributes(const Table *fromtable, Table *totable)
{
  PROC
  for(int i = 0; i < NUMBER_OF; i++)
    if(fromtable->hasAttribute(attributeName(static_cast<EvAtt>(i))))
      setEventAttribute(totable, static_cast<EvAtt>(i));
  propagateRejectionMasks(fromtable, totable);
  ENDPROC
}

const vector<string> EventAttributes::names()
{
  PROC
  map<string, EvAtt> ea = evatt_map();
  typedef map<string, EvAtt>::const_iterator CI;
  vector<string> v(ea.size());
  unsigned j = 0;
  for(CI i = ea.begin(); i != ea.end(); i++){
    SAS_DEBUG(cout << "EventAttributes::DEBUG " << (*i).first << endl;)
    v[j++] = (*i).first;
  }
  const vector<string> vc(v.begin(), v.end());
  RETURN vc;
}

EvAtt EventAttributes::lookup(const string & name)
{
  PROC
  map<string, EvAtt> ea = evatt_map();
  if(ea.find(name) == ea.end())
    Sas::error("EventAttributeDoesNotExist", "The event attribute " + name + " does not exist.");
  RETURN ea[name];
}


bool EventAttributes::eventAttributeIsSet(const Table *table, const EvAtt n)
{
  PROC
  RETURN table->hasAttribute(attributeName(n));
}
  
bool EventAttributes::eventAttributeIsSet(int & v, const EvAtt n)
{
  PROC
  int x = 0;
  setEventAttribute(x, n);
  RETURN (v & x);
}
