!+
! Module: addColumnOverwrite
!
! Authors:  Peter Videler - ESTEC/ESA
!          
! Description: Add new column to existing table and overwrite if exists
! 
! Comments:
! 
! $Id: dalutilities_mod.f90,v 1.3 2000/02/10 17:44:13 pvideler Exp $
!- 

module dalutilities_mod
  use dal
  use errorHandling

  implicit none

   ! ......................................................
  private 
  public addColumnOverwrite
  public moveAttribute              ! added PV 25-11-99
  public copyAttribute              ! added PV 25-11-99
  public moveAllAttributes          ! added PV 06-12-99
  public copyAllAttributes          ! added PV 06-12-99
  public moveSomeAttributes          ! added PV 10-02-2000
  public copySomeAttributes          ! added PV 10-02-2000
  ! ......................................................
    
contains
 
 !---------------------------------------------------------------------
  subroutine copySomeAttributes(from, to)
  !---------------------------------------------------------------------
  type(attributableT) :: from, to 

    call copyAttribute(attribute(from,"TELESCOP"), to)
    call copyAttribute(attribute(from,"INSTRUME"), to)
    call copyAttribute(attribute(from,"OBS_ID")  , to)
    call copyAttribute(attribute(from,"EXP_ID")  , to)
    call copyAttribute(attribute(from,"DATE-OBS"), to)
    call copyAttribute(attribute(from,"DATE-END"), to)

   end subroutine copySomeAttributes

 !---------------------------------------------------------------------
  subroutine moveSomeAttributes(from, to)
  !---------------------------------------------------------------------
  type(attributableT) :: from, to 

    call moveAttribute(attribute(from,"TELESCOP"), to)
    call moveAttribute(attribute(from,"INSTRUME"), to)
    call moveAttribute(attribute(from,"OBS_ID")  , to)
    call moveAttribute(attribute(from,"EXP_ID")  , to)
    call moveAttribute(attribute(from,"DATE-OBS"), to)
    call moveAttribute(attribute(from,"DATE-END"), to)

   end subroutine moveSomeAttributes

  !---------------------------------------------------------------------
  subroutine moveAllAttributes(from, to)
  !---------------------------------------------------------------------
  ! example: Move all attributes from mytble to myset
  !          call moveAllAttributes(attributable(mytbl), attributable(myset))

  type(attributableT) :: from, to 
  integer :: i, n

  n =  numberOfAttributes(from) 
  do i = 0, n-1
      ! write(*,*) 'moving attribute named:', name(Attribute(from, 0))
     call moveAttribute(Attribute(from, 0), to)
  enddo

  end subroutine

  !---------------------------------------------------------------------
  subroutine copyAllAttributes(from, to)
  !---------------------------------------------------------------------
  ! example: Copy all attributes from mytble to myset
  !          call copyAllAttributes(attributable(mytbl), attributable(myset))

  type(attributableT) :: from, to 
  integer :: i, n

  n =  numberOfAttributes(from) 
  do i = 0, n-1
     ! write(*,*) 'copying attribute named:', name(Attribute(from, i))
     call copyAttribute(Attribute(from, i), to)
  enddo

  end subroutine

  !---------------------------------------------------------------------
  subroutine moveAttribute(from, to)
  !---------------------------------------------------------------------
  ! example: Moves attribute "DATE" from mytbl to primary header of myset    
  !          call  moveAttribute(attribute(mytbl,"DATE"), attributable(myset))
  type(attributableT) :: to 
  type(attributeT)    :: from 

  call setAttribute(to, from)
  call deleteAttribute(parent(from), name(from))
  end subroutine

  !---------------------------------------------------------------------
  subroutine copyAttribute(from, to)
  !---------------------------------------------------------------------
  ! example: Copy attribute "DATE" from mytbl to primary header of myset    
  !           call copyAttribute(attribute(mytbl,"DATE"), attributable(myset))
  type(attributableT) ::  to 
  type(attributeT)     :: from 

  call setAttribute(to, from)
  end subroutine 

!---------------------------------------------------------------------
 function addColumnOverwrite(tbl, colname, itype, units) result(col)
    !---------------------------------------------------------------------

    type (tableT) ,   intent(in)              :: tbl
    type (columnT)                            :: col
    integer          , intent(in)             :: itype
    character (len=*), intent(in)             :: colname
    character (len=*), intent(in), optional   :: units

    if (hasColumn(tbl, colname)) then
       call message(appMsg, Verbose,'Overwriting existing '//colname//' column')
       col = Column(tbl, colname, MODIFY)
    else
       call message(appMsg, Verbose,'Adding column '//colname)
       if (present(units)) then
          col = addColumn(tbl, colname, itype, units)
       else
          col = addColumn(tbl, colname, itype)
       endif
    endif

  end function 
 
end module 












