! Module: eventAttributes
! Author: Giuseppe Vacanti, European Space Agency
!
! Description: SAS event flags, as per note GV/XMM/1999-01 [Issue 1.3]
!
! IMPORTANT: The flag definitions in this file must match those in EventAttributes.{h,cc}.
!
module eventAttributes
  use ErrorHandling
  use Dal
  implicit none
  private

  public :: setEventAttribute
  public :: copyEventAttributes
  public :: setEmosRejectionMask
  public :: setEpnRejectionMask
  public :: setRgsRejectionMask
! SCR 198
  public :: setEmosSpectralMask
  public :: eventAttributeIsSet

!  integer, parameter :: k10 = selected_int_kind(10)

  character(len=*), public, parameter :: EVATT_COLUMN_NAME = 'FLAG'

! Generate with:
!
!./testevattcc  | awk '{printf("integer(kind=int32), public, parameter :: EVATT_%s = ibset(0_int32,%d)\n",$4,$8)}'
!
integer(kind=int32), public, parameter :: EVATT_BAD_E3E4 = ibset(0_int32,29)
integer(kind=int32), public, parameter :: EVATT_BAD_SHAPE = ibset(0_int32,19)
integer(kind=int32), public, parameter :: EVATT_BELOW_ACCEPTANCE = ibset(0_int32,27)
integer(kind=int32), public, parameter :: EVATT_BELOW_THRESH = ibset(0_int32,26)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_BADCOL = ibset(0_int32,9)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_BADROW = ibset(0_int32,10)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_BRIGHTPIX = ibset(0_int32,6)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_CCD_BORDER = ibset(0_int32,1)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_CCD_WINDOW = ibset(0_int32,2)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_DEADPIX = ibset(0_int32,8)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_FLICKERINGPIX = ibset(0_int32,7)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_NODE_BOUNDARY = ibset(0_int32,3)
integer(kind=int32), public, parameter :: EVATT_CLOSE_TO_ONBOARD_BADPIX = ibset(0_int32,5)
integer(kind=int32), public, parameter :: EVATT_COSMIC_RAY = ibset(0_int32,19)
integer(kind=int32), public, parameter :: EVATT_DIAGONAL = ibset(0_int32,0)
integer(kind=int32), public, parameter :: EVATT_EVLIST_LOCAL = ibset(0_int32,31)
integer(kind=int32), public, parameter :: EVATT_FLICKERING = ibset(0_int32,15)
integer(kind=int32), public, parameter :: EVATT_INVALID_PATTERN = ibset(0_int32,0)
integer(kind=int32), public, parameter :: EVATT_IN_BAD_FRAME = ibset(0_int32,17)
integer(kind=int32), public, parameter :: EVATT_IN_SPOILED_FRAME = ibset(0_int32,11)
integer(kind=int32), public, parameter :: EVATT_LOEN_NOISE = ibset(0_int32,27)
integer(kind=int32), public, parameter :: EVATT_MIP_ASSOCIATED = ibset(0_int32,20)
integer(kind=int32), public, parameter :: EVATT_NEXT_TO_BADPIX = ibset(0_int32,24)
integer(kind=int32), public, parameter :: EVATT_NEXT_TO_CCD_BORDER = ibset(0_int32,23)
integer(kind=int32), public, parameter :: EVATT_NEXT_TO_OFFSET_COLUMN = ibset(0_int32,4)
integer(kind=int32), public, parameter :: EVATT_ONBOARD_SER = ibset(0_int32,12)
integer(kind=int32), public, parameter :: EVATT_ON_BADOFFSET = ibset(0_int32,13)
integer(kind=int32), public, parameter :: EVATT_ON_BADPIX = ibset(0_int32,21)
integer(kind=int32), public, parameter :: EVATT_ON_BADROW = ibset(0_int32,28)
integer(kind=int32), public, parameter :: EVATT_ON_NODE0 = ibset(0_int32,3)
integer(kind=int32), public, parameter :: EVATT_ON_NODE1 = ibset(0_int32,4)
integer(kind=int32), public, parameter :: EVATT_ON_NODE_INTERFACE = ibset(0_int32,20)
integer(kind=int32), public, parameter :: EVATT_ON_OFFSET_COLUMN = ibset(0_int32,3)
integer(kind=int32), public, parameter :: EVATT_ON_WINDOW_BORDER = ibset(0_int32,25)
integer(kind=int32), public, parameter :: EVATT_OUTSIDE_GTI = ibset(0_int32,18)
integer(kind=int32), public, parameter :: EVATT_OUTSIDE_THRESHOLDS = ibset(0_int32,26)
integer(kind=int32), public, parameter :: EVATT_OUT_OF_CCD_WINDOW = ibset(0_int32,25)
integer(kind=int32), public, parameter :: EVATT_OUT_OF_FOV = ibset(0_int32,16)
integer(kind=int32), public, parameter :: EVATT_REJECTED_BY_GATTI = ibset(0_int32,22)
integer(kind=int32), public, parameter :: EVATT_SECONDARY = ibset(0_int32,22)
integer(kind=int32), public, parameter :: EVATT_TRAILING = ibset(0_int32,23)
integer(kind=int32), public, parameter :: EVATT_UNDERSHOOT = ibset(0_int32,30)
integer(kind=int32), public, parameter :: EVATT_UNKNOWN_RAWX_PATTERN = ibset(0_int32,12)

  ! these are the instrument masks
  integer(kind=int32), public, parameter :: EVATT_RGS_REJECTION_MASK         = EVATT_ON_BADPIX + &
       EVATT_NEXT_TO_BADPIX + &
       EVATT_BAD_SHAPE + &
       EVATT_ON_WINDOW_BORDER + &
       EVATT_ON_NODE_INTERFACE + &
       EVATT_BELOW_ACCEPTANCE + &
       EVATT_IN_BAD_FRAME + &
       EVATT_OUTSIDE_GTI
  
  integer(kind=int32), public, parameter :: EVATT_EMOS_REJECTION_MASK        = EVATT_OUT_OF_FOV + &
       EVATT_IN_BAD_FRAME + &
       EVATT_COSMIC_RAY + &
       EVATT_ON_BADPIX + &
       EVATT_REJECTED_BY_GATTI + &
       EVATT_OUT_OF_CCD_WINDOW + &
       EVATT_OUTSIDE_THRESHOLDS + &
       EVATT_ON_BADROW + &
       EVATT_BAD_E3E4 + &
       EVATT_UNDERSHOOT + &
       EVATT_FLICKERING + &
	   EVATT_ON_BADOFFSET
  
  integer(kind=int32), public, parameter :: EVATT_EPN_REJECTION_MASK         = EVATT_IN_BAD_FRAME + &
       EVATT_COSMIC_RAY + &
       EVATT_ON_BADPIX + &
       EVATT_SECONDARY + &
       EVATT_TRAILING + &
       EVATT_MIP_ASSOCIATED + &
       EVATT_BELOW_THRESH + &
       EVATT_LOEN_NOISE
  
! SCR 198
  integer(kind=int32), public, parameter :: EMOS_SPECTRAL_MASK               = not(EVATT_CLOSE_TO_DEADPIX)

  integer(kind=int32), public, parameter :: EVATT_CLEAR_ALL_FLAGS            = 0      
  
  character(len=*), parameter :: errorMessage  = "Invalid event attributes library"
  
contains

  subroutine setEventAttribute(table, mask)
    type(TableT), intent(in) :: table
    integer(kind=int32), intent(in) :: mask
    call error( "", errorMessage )
  end subroutine setEventAttribute
  
  subroutine copyEventAttributes(fromtable, totable)
    type(TableT), intent(in) :: fromtable, totable
    call error( "", errorMessage )
  end subroutine copyEventAttributes

  subroutine setEmosRejectionMask(table)
    type(TableT), intent(in) :: table
    call error( "", errorMessage )
  end subroutine setEmosRejectionMask

  subroutine setEpnRejectionMask(table)
    type(TableT), intent(in) :: table
    call error( "", errorMessage )
  end subroutine setEpnRejectionMask

  subroutine setRgsRejectionMask(table)
    type(TableT), intent(in) :: table
    call error( "", errorMessage )
  end subroutine setRgsRejectionMask

! SCR 198
  subroutine setEmosSpectralMask(table)
    type(TableT), intent(in) :: table
    call error( "", errorMessage )
  end subroutine setEmosSpectralMask

  function eventAttributeIsSet(table, mask) 
    type(TableT), intent(in) :: table
    integer(kind=int32), intent(in) :: mask
    logical :: eventAttributeIsSet
    call error( "", errorMessage )
  end function eventAttributeIsSet
  
end module eventAttributes


