//
// Class implementation
//
// Giuseppe Vacanti, XMM-SOC ESA, 1998 November 30
//

// C++
#include <string>
#include <list>
#include <vector>
#include <cerrno>
/* AI gcc4 porting
 * ´strerror' was not declared in this scope
 */
#include <string.h>

// C
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <glob.h>

// SAS
#include "Tracer.h"
#include "errstr.h"
#include "stringutil.h"

// This aplication
#include "DirEntry.h"
#include "DirEntryFactory.h"
#include "Directory.h"

// Make sure we get the POSIX behavior
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE 1
#endif


Directory::Directory(const string name, const string pattern, const bool recurse)
  :  _isOk(true),
     _name(name),
     _pattern(pattern),
     _separator("|"),
     _recurse(recurse)
{
  PROC
  init();
  ENDPROC
}

Directory::Directory(const DirEntry * d, const string pattern, const bool recurse)
  :  _isOk(d->isDir()),
     _name(d->name()),
     _pattern(pattern),
     _separator("|"),
     _recurse(recurse)
{
  PROC
  init();
  ENDPROC
}

void Directory::init()
{
  PROC
  if (!_name.empty() && _name[_name.length() - 1] != '/')
    _name += "/";
  scan();
  ENDPROC
}

bool Directory::isOk() const {
  PROC
  RETURN _isOk;
}

string Directory::name() const {
  PROC
  RETURN _name;
}

void Directory::print(ostream & os) const
{
  PROC
  os << "Directory: " + name() << endl;
  for(Directory::const_iterator i = begin(); i != end(); i++)
    os << ">> " << **i << endl;
  ENDPROC
}


void Directory::filter(const string pattern) {
  PROC
  _pattern = pattern;
  reset();
  scan();
  ENDPROC
}

string Directory::filter() const {
  PROC
  RETURN _pattern;
}

void Directory::recurse(const bool dorecurse)
{
  PROC
  _recurse = dorecurse;
  reset();
  scan();
  ENDPROC
}

bool Directory::recurse() const
{
  PROC
  RETURN _recurse;
}

void Directory::scan() {
  PROC
  glob_t pglob;
  vector<string> search;
  if(_pattern.empty())
    search.push_back(_name + string("*"));
  else {
    search = breakup(_pattern, _separator);
    for(vector<string>::iterator i = search.begin(); i != search.end(); i++){
      *i = _name + *i;
    }
  }
  for(unsigned int j=0; j < search.size(); j++){
    errno = 0;
    int greturn = glob(search[j].c_str(), GLOB_MARK, NULL, &pglob);
    if(greturn == 0){
      // The cast is needed because on Linux gl_pathc is int. The Linux
      // man page says this is compliant with the proposed POSIX 2
      // standard. However POSIX 1 seems to say that unsigned long is
      // the return type (it is on Solaris).
      for (unsigned long i = 0; i < static_cast<unsigned int>(pglob.gl_pathc); i++) {
	DirEntry * e = dirEntryFactory->dirEntry(pglob.gl_pathv[i]);
	push_back(e);
      }
      // There would be a better way to do this, but GLOB_NOMATCH does
      // not exist on Mac OS X

      //     switch(glob(search[j].c_str(), GLOB_MARK, NULL, &pglob)) {
      //     case 0:
      //      -- GOOD
      //      break;
      //     case GLOB_NOMATCH:
      //      -- NO MATCH, but still good
      //       break;
      //     default:
      //      -- ERROR

    } else if(errno != 0 && pglob.gl_pathc != 0) {
      _isOk = false;
      errstr << "Error " << errno << ": " << strerror(errno) <<
	" while globbing " << search[j].c_str() << Sas::warning("directoryGlob");
    }
  }
  globfree(&pglob);
  //
  // This is the algorithm for recursiove
  //
  //  - rescan the current directory without any filtering or recursion
  //  - identify all subdirectories
  //  - scan each subdirectory with the current directory's filter
  //  - push all the entries on the list
  //
  if(recurse()){
    Directory dd(name());
    for(Directory::const_iterator i = dd.begin(); i != dd.end(); i++){
      if((*i)->isDir()){
	Directory sd(*i, filter(), recurse());
	for(Directory::const_iterator j = sd.begin(); j != sd.end(); j++)
	  push_back(*j);
      }
    }
  }
  ENDPROC
}

void Directory::reset() {
  PROC
  clear();
  ENDPROC
}

ostream & operator<<(ostream &os, const Directory &d)
{
  PROC
  d.print(os);
  RETURN os;
}

