module transform_mod

! $Id: transform_mod.f90,v 1.1 1999/04/01 12:22:36 pvideler Exp $
!+
! transform_mod.f90	(c) 1996 European Space Agency
!
! AUTHOR:	Marco Beijersbergen
! VERSION:	1.06
! DESCRIPTION:	Geometrical transformation object, using 4x4 matrices
!--
! INCLUDED FILE & MODULES:
!		matrix4_mod
!
! COMMENTS:
!
! MODIFICATIONS:
!    VERSION	DATE	AUTHOR	DESCRIPTION OF MOD.
!      1.05	6-8-96	MBe	TransRot added
!      1.06	8-1-99	PVi	Taken from SciSim. kr replaced by double
!                               private/public added
!+

  use types
  use matrix4_mod

 !......................................................................
  private
  public  ::              &
     transform,           &
     transformInit,       &
     transformTranslate,  &
     transformRotate,     &
     transformTransRot
 !......................................................................

  type Transform
    logical translated, rotated
    real(double), dimension(4,4) :: transform, inverse
  end type Transform


contains

  subroutine transformInit(t)
    type(Transform), intent(out) :: t

    t%rotated = .false.
    t%translated = .false.
    t%transform = matrix4Unity()
    t%inverse = matrix4Unity()
    
  end subroutine transformInit

  subroutine transformTranslate(t,v)
    type(Transform), intent(inout) :: t
    real(double), dimension(3), intent(in) :: v

    real(double), dimension(4,4) :: m

    if (any(v /= 0d0)) then
      m = matrix4Translation(v)
      t%transform  = matmul(t%transform,m)
      m = matrix4Translation(-v)
      t%inverse = matmul(m,t%inverse)
      t%translated = .true.
    end if

  end subroutine transformTranslate

  subroutine transformRotate(t,v,radians)
    type(Transform), intent(inout) :: t
    real(double), dimension(3), intent(in) :: v
    real(double), intent(in) :: radians

    real(double), dimension(4,4) :: m

    if (radians /= 0d0) then
      m = matrix4Rotation(v,radians)
      t%transform = matmul(t%transform,m)
      t%inverse = matmul(matrix4Inverse(m),t%inverse)
      t%rotated = .true.
    end if

  end subroutine transformRotate

  subroutine transformTransRot(t, distance, radians)
    type(Transform), intent(inout) :: t
    real(double), dimension(3), intent(in) :: distance, radians

    real(double), dimension(3) :: axis

    call transformTranslate(t,distance)
    axis = (/1d0, 0d0, 0d0/)
    call transformRotate(t,axis,radians(1))
    axis = (/0d0, 1d0, 0d0/)
    call transformRotate(t,axis,radians(2))
    axis = (/0d0, 0d0, 1d0/)
    call transformRotate(t,axis, radians(3))

  end subroutine transformTransRot

end module transform_mod

